
#include <math.h>
#include <gtk/gtk.h>

typedef struct _VnAllocator VnAllocator;
typedef struct _Item Item;

typedef void (*VnAllocatorShelfFunc) (VnAllocator * self, gpointer user_data);
typedef void (*VnAllocatorBoxFunc) (VnAllocator * self, gpointer user_data, Item * item, gint amount);

struct _Item
{
	gchar * name;
	gint box_height;
	gint box_width;
	gint amount;	
};

struct _VnAllocator
{
	Item * items;
	gint max_amount;
	VnAllocatorShelfFunc shelf_func;
	VnAllocatorBoxFunc box_func;
	gpointer user_data;

	gint ntrays;
	gint tray_width;
	gint tray_height;
	gint top_tray_height;

	gint first_shelf_box;
	gint current_shelf;
	gint current_tray;
	gint column_width;
	gint last_box_width;
	gint tray_x;
	gint tray_y;
};

void
vn_allocator_add_shelf (VnAllocator * self)
{
	self->current_shelf++;
	self->first_shelf_box = TRUE;
	
	if (self->shelf_func)
		self->shelf_func (self, self->user_data);
}

void
vn_allocator_add_tray (VnAllocator * self)
{
	if (self->current_tray <= 0)
	{
		vn_allocator_add_shelf (self);
		self->current_tray = self->ntrays - 1;
	}
	else
		self->current_tray--;
		
	self->tray_x = 0;
}

void
vn_allocator_add_column (VnAllocator * self, gint width)
{
	if (self->tray_x + self->column_width + width > self->tray_width
	|| self->current_tray == -1)
		vn_allocator_add_tray (self);
	else
		self->tray_x += self->column_width;

	self->tray_y = 0;
	self->column_width = width;
	self->last_box_width = width;
}

void
vn_allocator_add_box (VnAllocator * self, Item * i, gint amount)
{
	gint tray_height = self->tray_height;
	
	if (self->current_tray == self->ntrays - 1)
		tray_height = self->top_tray_height;

	if (self->tray_y + i->box_height > tray_height
	|| i->box_width > self->last_box_width)
		vn_allocator_add_column (self, i->box_width);

	if (self->box_func)
		self->box_func (self, self->user_data, i, amount);

	self->tray_y += i->box_height;
	
	if (i->box_width < self->last_box_width)
		self->last_box_width = i->box_width;
}

gint
vn_allocator_run (VnAllocator * self)
{
	Item * i;
	gint amount;

	self->first_shelf_box = FALSE;
	self->current_shelf = -1;
	self->current_tray = -1;
	self->column_width = 0;
	self->last_box_width = 0;
	self->tray_x = 0;
	self->tray_y = 0;

	for (i = self->items; i->name; i++)
	for (amount = 0; amount < i->amount; amount++)
	{
		if (amount == 0
		&& self->max_amount > 0
		&& i->amount > self->max_amount)
			break;
			
		vn_allocator_add_box (self, i, amount);

		self->first_shelf_box = FALSE;
	}
	
	return self->current_shelf + 1;
}
