/*
 * Copyright (C) 2012 - Juan Ferrer Toribio
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SQL_OBJECT_H
#define SQL_OBJECT_H

#include <gvn/gvn.h>
#include "sql-param-object.h"
#include "sql-param-list.h"

#define SQL_TYPE_OBJECT				(sql_object_get_type ())
#define SQL_OBJECT(obj)				(G_TYPE_CHECK_INSTANCE_CAST (obj, SQL_TYPE_OBJECT, SqlObject))
#define SQL_IS_OBJECT(obj)			(G_TYPE_CHECK_INSTANCE_TYPE (obj, SQL_TYPE_OBJECT))
#define SQL_OBJECT_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST (klass, SQL_TYPE_OBJECT, SqlObjectClass))
#define SQL_OBJECT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS (obj, SQL_TYPE_OBJECT, SqlObjectClass))

typedef struct _SqlObject SqlObject;
typedef struct _SqlObjectClass SqlObjectClass;

typedef gboolean	(* SqlObjectIsReadyFunc)		(SqlObject * obj);
typedef void		(* SqlObjectFindHoldersFunc)	(); // (SqlObject * obj, SqlBatch * holders);
typedef void		(* SqlRenderFunc)				(); // (SqlObject * obj, SqlRender * render, SqlBatch * batch);

struct _SqlObject
{
	GInitiallyUnowned parent;
};

struct _SqlObjectClass
{
	/* <private> */
	GInitiallyUnownedClass parent;
	SqlRenderFunc render;
	SqlObjectIsReadyFunc is_ready;
	SqlObjectFindHoldersFunc find_holders;
};

#include "sql-render.h"
#include "sql-batch.h"

GType		sql_object_get_type			();
void		sql_object_render			(SqlObject * obj, SqlRender * render, SqlBatch * batch);
gboolean	sql_object_is_ready			(SqlObject * obj);
void		sql_object_get_holders		(SqlObject * obj, SqlBatch * batch);

void		sql_object_set				(SqlObject * obj, const gchar * property, SqlObject * set);
SqlObject *	sql_object_get				(SqlObject * obj, const gchar * property);
void		sql_object_add_child		(SqlObject * obj, const gchar * property, SqlObject * child);
void		sql_object_remove_child		(SqlObject * obj, const gchar * property, guint n);
void		sql_object_changed			(SqlObject * obj);

gpointer	sql_object_add				(gpointer obj, gpointer child);
void		sql_object_remove			(gpointer obj, gpointer child);

#endif