/*
 * Copyright (C) 2012 - Juan Ferrer Toribio
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "vn-column-entry.h"

G_DEFINE_TYPE (VnColumnEntry, vn_column_entry, VN_TYPE_COLUMN);

VnColumn * vn_column_entry_new ()
{
	return g_object_new (VN_TYPE_COLUMN_ENTRY, NULL);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++ Private

static void vn_column_entry_cb_edited (GtkCellRendererText * cell,
	const gchar * path, gchar * text, VnColumnEntry * self)
{
	GValue value = G_VALUE_INIT;

	if (g_strcmp0 (text, ""))
	{
		g_value_init (&value, G_TYPE_STRING);
		g_value_set_string (&value, text);
	}
	else
		g_value_init (&value, GVN_TYPE_NULL);

	VN_COLUMN_GET_CLASS (self)->value_changed (VN_COLUMN (self), path, &value);
	g_value_unset (&value);
}

static void vn_column_entry_set_editable (VnColumn * self, gboolean editable)
{
	g_object_set (self->cell, "editable", editable, NULL);

	if (editable)
		g_signal_connect (self->cell, "edited",
			G_CALLBACK (vn_column_entry_cb_edited), self);
	else
		g_signal_handlers_disconnect_by_func (self->cell,
			vn_column_entry_cb_edited, self);
}

static void vn_column_entry_set_value (VnColumnEntry * self, GtkTreeModel * model,
	GtkTreeIter * iter, GObject * cell, const GValue * value)
{
	GValue new_value = G_VALUE_INIT;
	gvn_value_to_format_string (value, self->format, &new_value);
	g_object_set_property (cell, "text", &new_value);
	g_value_unset (&new_value);
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++ Properties

enum
{
	 PROP_FORMAT = 1
};

static void vn_column_entry_set_property (VnColumnEntry * self, guint id,
	const GValue * value, GParamSpec * pspec)
{
	switch (id)
	{
		case PROP_FORMAT:
			g_free (self->format);
			self->format = g_value_dup_string (value);
			break;
		default:		
			G_OBJECT_WARN_INVALID_PROPERTY_ID (self, id, pspec);
	}
}

static void vn_column_entry_get_property (VnColumnEntry * self, guint id,
	GValue * value, GParamSpec * pspec)
{	
	switch (id)
	{
		case PROP_FORMAT:
			g_value_set_string (value, self->format);
			break;
		default:		
			G_OBJECT_WARN_INVALID_PROPERTY_ID (self, id, pspec);
	}
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++ Class

static void vn_column_entry_init (VnColumnEntry * self)
{
	GtkCellRenderer * cell = gtk_cell_renderer_text_new ();
	VN_COLUMN_GET_CLASS (self)->set_renderer (VN_COLUMN (self), cell); 
	self->format = NULL;
}

static void vn_column_entry_finalize (VnColumnEntry * self)
{
	g_free (self->format);
	G_OBJECT_CLASS (vn_column_entry_parent_class)->finalize (G_OBJECT (self));
}

static void vn_column_entry_class_init (VnColumnEntryClass * klass)
{
	GObjectClass * k = G_OBJECT_CLASS (klass);
	k->finalize = (GObjectFinalizeFunc) vn_column_entry_finalize;
	k->set_property = (GObjectSetPropertyFunc) vn_column_entry_set_property;
	k->get_property = (GObjectGetPropertyFunc) vn_column_entry_get_property;
	VN_COLUMN_CLASS (klass)->set_value = (VnColumnSetValueFunc) vn_column_entry_set_value;
	VN_COLUMN_CLASS (klass)->set_editable = (VnColumnSetEditableFunc) vn_column_entry_set_editable;

	g_object_class_install_property (k, PROP_FORMAT,
		g_param_spec_string ("format"
			,_("Format")
			,_("The format string describing the output of the entry.")
			,NULL
			,G_PARAM_CONSTRUCT | G_PARAM_READWRITE
	));
}
